import { Request, Response } from 'express';
import validator from 'joi'; // DOCS: https://joi.dev/api
import log from '../../../handlers/log.js';


const schema_get = validator.object({
	sort: validator.string().valid('id', 'name', 'phone', 'comment').default('id'),
	order: validator.string().valid('asc', 'desc').default('asc'),
	search: validator.string().min(3).max(20), // TODO: Check if * or ** or *** -> Due to crashes..
	id: validator.number().positive().precision(0),
	count: validator.boolean()
}).nand('id', 'search'); // Allow id or search. not both.

const schema_post = validator.object({
	name: validator.string().min(1).max(32).required(),
	phone: validator.string().pattern(new RegExp('^\\+(\\d{1,3})\\s*(?:\\(\\s*(\\d{2,5})\\s*\\)|\\s*(\\d{2,5})\\s*)\\s*(\\d{5,15})$')).required(),
	comment: validator.string().max(64),
})

const schema_patch = validator.object({
	sort: validator.string().valid('id', 'name', 'phone', 'comment').default('id'),
	order: validator.string().valid('asc', 'desc').default('asc'),
	search: validator.string().min(3).max(20), // TODO: Check if * or ** or *** -> Due to crashes..
	id: validator.number().positive().precision(0),
	count: validator.boolean()
}).nand('id', 'search'); // Allow id or search. not both.

const schema_del = validator.object({
	sort: validator.string().valid('id', 'name', 'phone', 'comment').default('id'),
	order: validator.string().valid('asc', 'desc').default('asc'),
	search: validator.string().min(3).max(20), // TODO: Check if * or ** or *** -> Due to crashes..
	id: validator.number().positive().precision(0),
	count: validator.boolean()
}).nand('id', 'search'); // Allow id or search. not both.

// Describe all schemas
const schema_get_describe = schema_get.describe();
const schema_post_describe = schema_post.describe();
const schema_patch_describe = schema_patch.describe();
const schema_del_describe = schema_del.describe();


// GET route
export default async function get(req: Request, res: Response) {
	res.status(200).json({
		GET: schema_get_describe,
		POST: schema_post_describe,
		PATCH: schema_patch_describe,
		DELETE: schema_del_describe
	});
}

export { schema_get, schema_post, schema_patch, schema_del }
